@@ -43,8 +43,7 @@
 /**
  * GtkColumnView:
  *
- * `GtkColumnView` presents a large dynamic list of items using multiple columns
- * with headers.
+ * Presents a large dynamic list of items using multiple columns with headers.
  *
  * `GtkColumnView` uses the factories of its columns to generate a cell widget for
  * each column, for each visible item and displays them together as the row for
@@ -104,10 +103,10 @@
  *
  * # Accessibility
  *
- * `GtkColumnView` uses the %GTK_ACCESSIBLE_ROLE_TREE_GRID role, header title
- * widgets are using the %GTK_ACCESSIBLE_ROLE_COLUMN_HEADER role. The row widgets
- * are using the %GTK_ACCESSIBLE_ROLE_ROW role, and individual cells are using
- * the %GTK_ACCESSIBLE_ROLE_GRID_CELL role
+ * `GtkColumnView` uses the [enum@Gtk.AccessibleRole.tree_grid] role, header title
+ * widgets are using the [enum@Gtk.AccessibleRole.column_header] role. The row widgets
+ * are using the [enum@Gtk.AccessibleRole.row] role, and individual cells are using
+ * the [enum@Gtk.AccessibleRole.grid_cell] role
  */
 
 /* We create a subclass of GtkListView for the sole purpose of overriding
@@ -484,7 +483,7 @@ gtk_column_view_allocate (GtkWidget *widget,
                           int        baseline)
 {
   GtkColumnView *self = GTK_COLUMN_VIEW (widget);
-  int full_width, header_height, min, nat, x;
+  int full_width, header_height, min, nat, x, dx;
 
   x = gtk_adjustment_get_value (self->hadjustment);
   full_width = gtk_column_view_allocate_columns (self, width);
@@ -494,12 +493,15 @@ gtk_column_view_allocate (GtkWidget *widget,
     header_height = min;
   else
     header_height = nat;
+
+  dx = (_gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL) ? -x : width - full_width + x;
+
   gtk_widget_allocate (self->header, full_width, header_height, -1,
-                       gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (-x, 0)));
+                       gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (dx, 0)));
 
   gtk_widget_allocate (GTK_WIDGET (self->listview),
                        full_width, height - header_height, -1,
-                       gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (-x, header_height)));
+                       gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (dx, header_height)));
 
   gtk_adjustment_configure (self->hadjustment,  x, 0, full_width, width * 0.1, width * 0.9, width);
 }
@@ -941,7 +943,7 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
 
   /**
    * GtkColumnView::activate:
-   * @self: The `GtkColumnView`
+   * @self: The columnview
    * @position: position of item to activate
    *
    * Emitted when a row has been activated by the user, usually via activating
@@ -1069,7 +1071,9 @@ gtk_column_view_in_resize_rect (GtkColumnView       *self,
   gtk_column_view_column_get_allocation (column, NULL, &width);
   rect.size.width = width;
 
-  rect.origin.x += rect.size.width - DRAG_WIDTH / 2;
+  if (_gtk_widget_get_direction (GTK_WIDGET (self)) != GTK_TEXT_DIR_RTL)
+    rect.origin.x += rect.size.width;
+  rect.origin.x -= DRAG_WIDTH / 2;
   rect.size.width = DRAG_WIDTH;
 
   return graphene_rect_contains_point (&rect, &(graphene_point_t) { x, y});
@@ -1146,7 +1150,14 @@ header_drag_begin (GtkGestureDrag *gesture,
           gtk_column_view_column_set_fixed_width (column, size);
 
           self->drag_pos = i;
-          self->drag_x = start_x - size;
+          if (_gtk_widget_get_direction (GTK_WIDGET (self)) != GTK_TEXT_DIR_RTL)
+            self->drag_x = start_x - size;
+          else
+            {
+              int width = gtk_widget_get_width (GTK_WIDGET (self->header));
+
+              self->drag_x = width - (start_x + size);
+            }
           self->in_column_resize = TRUE;
 
           set_resize_cursor (self, TRUE);
@@ -1258,6 +1269,13 @@ update_column_resize (GtkColumnView *self,
 {
   GtkColumnViewColumn *column;
 
+  if (_gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL)
+    {
+      int width = gtk_widget_get_width (GTK_WIDGET (self->header));
+
+      x = width - x;
+    }
+
   column = g_list_model_get_item (G_LIST_MODEL (self->columns), self->drag_pos);
   gtk_column_view_column_set_fixed_width (column, MAX (x - self->drag_x, 0));
   g_object_unref (column);
@@ -1525,7 +1543,7 @@ gtk_column_view_new (GtkSelectionModel *model)
 
 /**
  * gtk_column_view_get_model:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Gets the model that's currently used to read the items displayed.
  *
@@ -1541,7 +1559,7 @@ gtk_column_view_get_model (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_model:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  * @model: (nullable) (transfer none): the model to use
  *
  * Sets the model to use.
@@ -1565,13 +1583,13 @@ gtk_column_view_set_model (GtkColumnView     *self,
 
 /**
  * gtk_column_view_get_columns:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Gets the list of columns in this column view.
  *
  * This list is constant over the lifetime of @self and can be used to
  * monitor changes to the columns of @self by connecting to the
- * ::items-changed signal.
+ * [signal@Gio.ListModel::items-changed] signal.
  *
  * Returns: (transfer none): The list managing the columns
  */
@@ -1585,11 +1603,10 @@ gtk_column_view_get_columns (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_show_row_separators:
- * @self: a `GtkColumnView`
- * @show_row_separators: %TRUE to show row separators
+ * @self: a columnview
+ * @show_row_separators: whether to show row separators
  *
- * Sets whether the list should show separators
- * between rows.
+ * Sets whether the list should show separators between rows.
  */
 void
 gtk_column_view_set_show_row_separators (GtkColumnView *self,
@@ -1607,12 +1624,11 @@ gtk_column_view_set_show_row_separators (GtkColumnView *self,
 
 /**
  * gtk_column_view_get_show_row_separators:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
- * Returns whether the list should show separators
- * between rows.
+ * Returns whether the list should show separators between rows.
  *
- * Returns: %TRUE if the list shows separators
+ * Returns: true if the list shows separators
  */
 gboolean
 gtk_column_view_get_show_row_separators (GtkColumnView *self)
@@ -1624,11 +1640,10 @@ gtk_column_view_get_show_row_separators (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_show_column_separators:
- * @self: a `GtkColumnView`
- * @show_column_separators: %TRUE to show column separators
+ * @self: a columnview
+ * @show_column_separators: whether to show column separators
  *
- * Sets whether the list should show separators
- * between columns.
+ * Sets whether the list should show separators between columns.
  */
 void
 gtk_column_view_set_show_column_separators (GtkColumnView *self,
@@ -1651,12 +1666,11 @@ gtk_column_view_set_show_column_separators (GtkColumnView *self,
 
 /**
  * gtk_column_view_get_show_column_separators:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
- * Returns whether the list should show separators
- * between columns.
+ * Returns whether the list should show separators between columns.
  *
- * Returns: %TRUE if the list shows column separators
+ * Returns: true if the list shows column separators
  */
 gboolean
 gtk_column_view_get_show_column_separators (GtkColumnView *self)
@@ -1668,9 +1682,8 @@ gtk_column_view_get_show_column_separators (GtkColumnView *self)
 
 /**
  * gtk_column_view_append_column:
- * @self: a `GtkColumnView`
- * @column: a `GtkColumnViewColumn` that hasn't been added to a
- *   `GtkColumnView` yet
+ * @self: a columnview
+ * @column: a column that hasn't been added to a `GtkColumnView` yet
  *
  * Appends the @column to the end of the columns in @self.
  */
@@ -1688,8 +1701,8 @@ gtk_column_view_append_column (GtkColumnView       *self,
 
 /**
  * gtk_column_view_remove_column:
- * @self: a `GtkColumnView`
- * @column: a `GtkColumnViewColumn` that's part of @self
+ * @self: a columnview
+ * @column: a column that's part of @self
  *
  * Removes the @column from the list of columns of @self.
  */
@@ -1733,9 +1746,9 @@ gtk_column_view_remove_column (GtkColumnView       *self,
 
 /**
  * gtk_column_view_insert_column:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  * @position: the position to insert @column at
- * @column: the `GtkColumnViewColumn` to insert
+ * @column: the column to insert
  *
  * Inserts a column at the given position in the columns of @self.
  *
@@ -1874,7 +1887,7 @@ gtk_column_view_get_list_view (GtkColumnView *self)
 
 /**
  * gtk_column_view_get_sorter:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Returns a special sorter that reflects the users sorting
  * choices in the column view.
@@ -1908,8 +1921,8 @@ gtk_column_view_get_sorter (GtkColumnView *self)
 
 /**
  * gtk_column_view_sort_by_column:
- * @self: a `GtkColumnView`
- * @column: (nullable): the `GtkColumnViewColumn` to sort by
+ * @self: a columnview
+ * @column: (nullable): the column to sort by
  * @direction: the direction to sort in
  *
  * Sets the sorting of the view.
@@ -1923,7 +1936,7 @@ gtk_column_view_get_sorter (GtkColumnView *self)
  * and [method@Gtk.ColumnViewColumn.set_sorter] has been called
  * on @column to associate a sorter with the column.
  *
- * If @column is %NULL, the view will be unsorted.
+ * If @column is unset, the view will be unsorted.
  */
 void
 gtk_column_view_sort_by_column (GtkColumnView       *self,
@@ -1944,8 +1957,8 @@ gtk_column_view_sort_by_column (GtkColumnView       *self,
 
 /**
  * gtk_column_view_set_single_click_activate:
- * @self: a `GtkColumnView`
- * @single_click_activate: %TRUE to activate items on single click
+ * @self: a columnview
+ * @single_click_activate: whether to activate items on single click
  *
  * Sets whether rows should be activated on single click and
  * selected on hover.
@@ -1966,12 +1979,12 @@ gtk_column_view_set_single_click_activate (GtkColumnView *self,
 
 /**
  * gtk_column_view_get_single_click_activate:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Returns whether rows will be activated on single click and
  * selected on hover.
  *
- * Returns: %TRUE if rows are activated on single click
+ * Returns: true if rows are activated on single click
  */
 gboolean
 gtk_column_view_get_single_click_activate (GtkColumnView *self)
@@ -1983,7 +1996,7 @@ gtk_column_view_get_single_click_activate (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_reorderable:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  * @reorderable: whether columns should be reorderable
  *
  * Sets whether columns should be reorderable by dragging.
@@ -2004,11 +2017,11 @@ gtk_column_view_set_reorderable (GtkColumnView *self,
 
 /**
  * gtk_column_view_get_reorderable:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Returns whether columns are reorderable.
  *
- * Returns: %TRUE if columns are reorderable
+ * Returns: true if columns are reorderable
  */
 gboolean
 gtk_column_view_get_reorderable (GtkColumnView *self)
@@ -2020,8 +2033,8 @@ gtk_column_view_get_reorderable (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_enable_rubberband:
- * @self: a `GtkColumnView`
- * @enable_rubberband: %TRUE to enable rubberband selection
+ * @self: a columnview
+ * @enable_rubberband: whether to enable rubberband selection
  *
  * Sets whether selections can be changed by dragging with the mouse.
  */
@@ -2041,11 +2054,11 @@ gtk_column_view_set_enable_rubberband (GtkColumnView *self,
 
 /**
  * gtk_column_view_get_enable_rubberband:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Returns whether rows can be selected by dragging with the mouse.
  *
- * Returns: %TRUE if rubberband selection is enabled
+ * Returns: true if rubberband selection is enabled
  */
 gboolean
 gtk_column_view_get_enable_rubberband (GtkColumnView *self)
@@ -2057,16 +2070,19 @@ gtk_column_view_get_enable_rubberband (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_row_factory:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  * @factory: (nullable): The row factory
  *
- * Sets the factory used for configuring rows. The factory must be for configuring
- * [class@Gtk.ColumnViewRow] objects.
+ * Sets the factory used for configuring rows.
  *
- * If this factory is not set - which is the default - then the defaults will be used.
+ * The factory must be for configuring [class@Gtk.ColumnViewRow] objects.
  *
- * This factory is not used to set the widgets displayed in the individual cells. For
- * that see [method@GtkColumnViewColumn.set_factory] and [class@GtkColumnViewCell].
+ * If this factory is not set - which is the default - then the defaults
+ * will be used.
+ *
+ * This factory is not used to set the widgets displayed in the individual
+ * cells. For that see [method@GtkColumnViewColumn.set_factory] and
+ * [class@GtkColumnViewCell].
  *
  * Since: 4.12
  */
@@ -2086,7 +2102,7 @@ gtk_column_view_set_row_factory (GtkColumnView      *self,
 
 /**
  * gtk_column_view_get_row_factory:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Gets the factory set via [method@Gtk.ColumnView.set_row_factory].
  *
@@ -2104,10 +2120,14 @@ gtk_column_view_get_row_factory (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_tab_behavior:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  * @tab_behavior: The desired tab behavior
  *
- * Sets the behavior of the <kbd>Tab</kbd> and <kbd>Shift</kbd>+<kbd>Tab</kbd> keys.
+ * Sets the <kbd>Tab</kbd> key behavior.
+ *
+ * This influences how the <kbd>Tab</kbd> and
+ * <kbd>Shift</kbd>+<kbd>Tab</kbd> keys move the
+ * focus in the columnview.
  *
  * Since: 4.12
  */
@@ -2127,7 +2147,7 @@ gtk_column_view_set_tab_behavior (GtkColumnView      *self,
 
 /**
  * gtk_column_view_get_tab_behavior:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Gets the behavior set for the <kbd>Tab</kbd> key.
  *
@@ -2145,7 +2165,7 @@ gtk_column_view_get_tab_behavior (GtkColumnView *self)
 
 /**
  * gtk_column_view_get_header_factory:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Gets the factory that's currently used to populate section headers.
  *
@@ -2163,13 +2183,13 @@ gtk_column_view_get_header_factory (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_header_factory:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  * @factory: (nullable) (transfer none): the factory to use
  *
- * Sets the `GtkListItemFactory` to use for populating the
+ * Sets the factory to use for populating the
  * [class@Gtk.ListHeader] objects used in section headers.
  *
- * If this factory is set to %NULL, the list will not show
+ * If this factory is set to `NULL`, the list will not show
  * section headers.
  *
  * Since: 4.12
@@ -2191,11 +2211,11 @@ gtk_column_view_set_header_factory (GtkColumnView      *self,
 
 /**
  * gtk_column_view_scroll_to:
- * @self: The columnview to scroll in
+ * @self: The columnview
  * @pos: position of the item. Must be less than the number of
  *   items in the view.
  * @column: (nullable) (transfer none): The column to scroll to
- *   or %NULL to not scroll columns.
+ *   or `NULL` to not scroll columns
  * @flags: actions to perform
  * @scroll: (nullable) (transfer full): details of how to perform
  *   the scroll operation or %NULL to scroll into view
@@ -2203,7 +2223,7 @@ gtk_column_view_set_header_factory (GtkColumnView      *self,
  * Scroll to the row at the given position - or cell if a column is
  * given - and performs the actions specified in @flags.
  *
- * This function works no matter if the listview is shown or focused.
+ * This function works no matter if the columnview is shown or focused.
  * If it isn't, then the changes will take effect once that happens.
  *
  * Since: 4.12
@@ -2236,4 +2256,3 @@ gtk_column_view_scroll_to (GtkColumnView       *self,
   else
     g_clear_pointer (&scroll, gtk_scroll_info_unref);
 }
-
