@@ -47,6 +47,7 @@
 #include <gsk/gskroundedrectprivate.h>
 #include <gsk/gsktransformprivate.h>
 
+#include <cairo-gobject.h>
 #include <glib/gi18n-lib.h>
 #include <gdk/gdkdmabufprivate.h>
 #include <gdk/gdkdmabuftextureprivate.h>
@@ -60,6 +61,7 @@
 #include "gtk/gtkdebug.h"
 #include "gtk/gtkbuiltiniconprivate.h"
 #include "gtk/gtkrendernodepaintableprivate.h"
+#include "gdk/gdkcairoprivate.h"
 
 #include "recording.h"
 #include "renderrecording.h"
@@ -233,6 +235,7 @@ enum
   PROP_DEBUG_NODES,
   PROP_HIGHLIGHT_SEQUENCES,
   PROP_SELECTED_SEQUENCE,
+  PROP_RECORD_EVENTS,
   LAST_PROP
 };
 
@@ -838,13 +841,14 @@ get_color2_texture (const GdkColor *color)
 
   texture = gdk_memory_texture_builder_build (builder);
 
+  g_object_unref (builder);
   g_bytes_unref (bytes);
 
   return texture;
 }
 
 static GdkTexture *
-get_linear_gradient_texture (gsize n_stops, const GskColorStop *stops)
+get_linear_gradient_texture (gsize n_stops, const GskColorStop2 *stops)
 {
   cairo_surface_t *surface;
   cairo_t *cr;
@@ -857,14 +861,7 @@ get_linear_gradient_texture (gsize n_stops, const GskColorStop *stops)
 
   pattern = cairo_pattern_create_linear (0, 0, 90, 0);
   for (i = 0; i < n_stops; i++)
-    {
-      cairo_pattern_add_color_stop_rgba (pattern,
-                                         stops[i].offset,
-                                         stops[i].color.red,
-                                         stops[i].color.green,
-                                         stops[i].color.blue,
-                                         stops[i].color.alpha);
-    }
+    gdk_cairo_pattern_add_color_stop_color (pattern, GDK_COLOR_STATE_SRGB, stops[i].offset, &stops[i].color);
 
   cairo_set_source (cr, pattern);
   cairo_pattern_destroy (pattern);
@@ -971,6 +968,14 @@ enum_to_nick (GType type,
   return v->value_nick;
 }
 
+static const char *
+hue_interpolation_to_string (GskHueInterpolation value)
+{
+  const char *name[] = { "shorter", "longer", "increasing", "decreasing" };
+
+  return name[value];
+}
+
 static void
 add_texture_rows (GListStore *store,
                   GdkTexture *texture)
@@ -1114,18 +1119,22 @@ populate_render_node_properties (GListStore    *store,
         const graphene_point_t *start = gsk_linear_gradient_node_get_start (node);
         const graphene_point_t *end = gsk_linear_gradient_node_get_end (node);
         const gsize n_stops = gsk_linear_gradient_node_get_n_color_stops (node);
-        const GskColorStop *stops = gsk_linear_gradient_node_get_color_stops (node, NULL);
+        const GskColorStop2 *stops = gsk_linear_gradient_node_get_color_stops2 (node);
+        GdkColorState *interpolation = gsk_linear_gradient_node_get_interpolation_color_state (node);
+        GskHueInterpolation hue_interpolation = gsk_linear_gradient_node_get_hue_interpolation (node);
         int i;
         GString *s;
         GdkTexture *texture;
 
         add_text_row (store, "Direction", "%.2f %.2f ⟶ %.2f %.2f", start->x, start->y, end->x, end->y);
+        add_text_row (store, "Interpolation", "%s", gdk_color_state_get_name (interpolation));
+        add_text_row (store, "Hue Interpolation", "%s", hue_interpolation_to_string (hue_interpolation));
 
         s = g_string_new ("");
         for (i = 0; i < n_stops; i++)
           {
             g_string_append_printf (s, "%.2f, ", stops[i].offset);
-            gdk_rgba_print (&stops[i].color, s);
+            gdk_color_print (&stops[i].color, s);
             g_string_append_c (s, '\n');
           }
 
@@ -1146,7 +1155,9 @@ populate_render_node_properties (GListStore    *store,
         const float hradius = gsk_radial_gradient_node_get_hradius (node);
         const float vradius = gsk_radial_gradient_node_get_vradius (node);
         const gsize n_stops = gsk_radial_gradient_node_get_n_color_stops (node);
-        const GskColorStop *stops = gsk_radial_gradient_node_get_color_stops (node, NULL);
+        const GskColorStop2 *stops = gsk_radial_gradient_node_get_color_stops2 (node);
+        GdkColorState *interpolation = gsk_radial_gradient_node_get_interpolation_color_state (node);
+        GskHueInterpolation hue_interpolation = gsk_radial_gradient_node_get_hue_interpolation (node);
         int i;
         GString *s;
         GdkTexture *texture;
@@ -1154,12 +1165,14 @@ populate_render_node_properties (GListStore    *store,
         add_text_row (store, "Center", "%.2f, %.2f", center->x, center->y);
         add_text_row (store, "Direction", "%.2f ⟶  %.2f", start, end);
         add_text_row (store, "Radius", "%.2f, %.2f", hradius, vradius);
+        add_text_row (store, "Interpolation", "%s", gdk_color_state_get_name (interpolation));
+        add_text_row (store, "Hue Interpolation", "%s", hue_interpolation_to_string (hue_interpolation));
 
         s = g_string_new ("");
         for (i = 0; i < n_stops; i++)
           {
             g_string_append_printf (s, "%.2f, ", stops[i].offset);
-            gdk_rgba_print (&stops[i].color, s);
+            gdk_color_print (&stops[i].color, s);
             g_string_append_c (s, '\n');
           }
 
@@ -1176,19 +1189,23 @@ populate_render_node_properties (GListStore    *store,
         const graphene_point_t *center = gsk_conic_gradient_node_get_center (node);
         const float rotation = gsk_conic_gradient_node_get_rotation (node);
         const gsize n_stops = gsk_conic_gradient_node_get_n_color_stops (node);
-        const GskColorStop *stops = gsk_conic_gradient_node_get_color_stops (node, NULL);
+        const GskColorStop2 *stops = gsk_conic_gradient_node_get_color_stops2 (node);
+        GdkColorState *interpolation = gsk_conic_gradient_node_get_interpolation_color_state (node);
+        GskHueInterpolation hue_interpolation = gsk_conic_gradient_node_get_hue_interpolation (node);
         gsize i;
         GString *s;
         GdkTexture *texture;
 
         add_text_row (store, "Center", "%.2f, %.2f", center->x, center->y);
         add_text_row (store, "Rotation", "%.2f", rotation);
+        add_text_row (store, "Interpolation", "%s", gdk_color_state_get_name (interpolation));
+        add_text_row (store, "Hue Interpolation", "%s", hue_interpolation_to_string (hue_interpolation));
 
         s = g_string_new ("");
         for (i = 0; i < n_stops; i++)
           {
             g_string_append_printf (s, "%.2f, ", stops[i].offset);
-            gdk_rgba_print (&stops[i].color, s);
+            gdk_color_print (&stops[i].color, s);
             g_string_append_c (s, '\n');
           }
 
@@ -1207,6 +1224,11 @@ populate_render_node_properties (GListStore    *store,
         PangoFontDescription *desc;
         GString *s;
         gchar *tmp;
+        cairo_scaled_font_t *sf;
+        cairo_font_options_t *options;
+        cairo_hint_style_t hint_style;
+        cairo_antialias_t antialias;
+        cairo_hint_metrics_t hint_metrics;
 
         desc = pango_font_describe ((PangoFont *)font);
         tmp = pango_font_description_to_string (desc);
@@ -1214,6 +1236,18 @@ populate_render_node_properties (GListStore    *store,
         g_free (tmp);
         pango_font_description_free (desc);
 
+        sf = pango_cairo_font_get_scaled_font (PANGO_CAIRO_FONT (font));
+        options = cairo_font_options_create ();
+        cairo_scaled_font_get_font_options (sf, options);
+        hint_style = cairo_font_options_get_hint_style (options);
+        antialias = cairo_font_options_get_antialias (options);
+        hint_metrics = cairo_font_options_get_hint_metrics (options);
+        cairo_font_options_destroy (options);
+
+        add_text_row (store, "Hint Style", "%s", enum_to_nick (CAIRO_GOBJECT_TYPE_HINT_STYLE, hint_style));
+        add_text_row (store, "Antialias", "%s", enum_to_nick (CAIRO_GOBJECT_TYPE_ANTIALIAS, antialias));
+        add_text_row (store, "Hint Metrics", "%s", enum_to_nick (CAIRO_GOBJECT_TYPE_HINT_METRICS, hint_metrics));
+
         s = g_string_sized_new (0);
         gsk_text_node_serialize_glyphs (node, s);
         add_text_row (store, "Glyphs", "%s", s->str);
@@ -2261,6 +2295,10 @@ gtk_inspector_recorder_get_property (GObject    *object,
       g_value_set_boolean (value, recorder->recording != NULL);
       break;
 
+    case PROP_RECORD_EVENTS:
+      g_value_set_boolean (value, recorder->record_events);
+      break;
+
     case PROP_DEBUG_NODES:
       g_value_set_boolean (value, recorder->debug_nodes);
       break;
@@ -2293,6 +2331,10 @@ gtk_inspector_recorder_set_property (GObject      *object,
       gtk_inspector_recorder_set_recording (recorder, g_value_get_boolean (value));
       break;
 
+    case PROP_RECORD_EVENTS:
+      recorder->record_events = g_value_get_boolean (value);
+      break;
+
     case PROP_DEBUG_NODES:
       gtk_inspector_recorder_set_debug_nodes (recorder, g_value_get_boolean (value));
       break;
@@ -2335,20 +2377,18 @@ gtk_inspector_recorder_class_init (GtkInspectorRecorderClass *klass)
   object_class->set_property = gtk_inspector_recorder_set_property;
   object_class->dispose = gtk_inspector_recorder_dispose;
 
-  props[PROP_RECORDING] =
-    g_param_spec_boolean ("recording", NULL, NULL,
-                          FALSE,
-                          G_PARAM_READWRITE);
-  props[PROP_DEBUG_NODES] =
-    g_param_spec_boolean ("debug-nodes", NULL, NULL,
-                          FALSE,
-                          G_PARAM_READWRITE);
-
+  props[PROP_RECORDING] = g_param_spec_boolean ("recording", NULL, NULL, FALSE, G_PARAM_READWRITE);
+  props[PROP_RECORD_EVENTS] = g_param_spec_boolean ("record-events", NULL, NULL, TRUE, G_PARAM_READWRITE);
+  props[PROP_DEBUG_NODES] = g_param_spec_boolean ("debug-nodes", NULL, NULL, FALSE, G_PARAM_READWRITE);
   props[PROP_HIGHLIGHT_SEQUENCES] = g_param_spec_boolean ("highlight-sequences", NULL, NULL, FALSE, G_PARAM_READWRITE);
   props[PROP_SELECTED_SEQUENCE] = g_param_spec_pointer ("selected-sequence", NULL, NULL, G_PARAM_READWRITE);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
+  gtk_widget_class_install_property_action (widget_class, "record.record-events", "record-events");
+  gtk_widget_class_install_property_action (widget_class, "record.debug-nodes", "debug-nodes");
+  gtk_widget_class_install_property_action (widget_class, "record.highlight-sequences", "highlight-sequences");
+
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gtk/libgtk/inspector/recorder.ui");
 
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorRecorder, box);
@@ -2380,6 +2420,8 @@ gtk_inspector_recorder_init (GtkInspectorRecorder *recorder)
   GtkSelectionModel *model;
   GtkColumnViewColumn *column;
 
+  recorder->record_events = TRUE;
+
   gtk_widget_init_template (GTK_WIDGET (recorder));
 
   factory = gtk_signal_list_item_factory_new ();
@@ -2475,7 +2517,6 @@ gtk_inspector_recorder_set_recording (GtkInspectorRecorder *recorder,
     {
       recorder->recording = gtk_inspector_start_recording_new ();
       recorder->start_time = 0;
-      recorder->record_events = TRUE;
       gtk_inspector_recorder_add_recording (recorder, recorder->recording);
     }
   else
@@ -2540,7 +2581,6 @@ gtk_inspector_recorder_record_render (GtkInspectorRecorder *recorder,
     }
 
   recording = gtk_inspector_render_recording_new (frame_time,
-                                                  gsk_renderer_get_profiler (renderer),
                                                   &(GdkRectangle) { 0, 0,
                                                     gdk_surface_get_width (surface),
                                                     gdk_surface_get_height (surface) },
@@ -2682,5 +2722,4 @@ gtk_inspector_recorder_set_selected_sequence (GtkInspectorRecorder *recorder,
   g_object_notify_by_pspec (G_OBJECT (recorder), props[PROP_SELECTED_SEQUENCE]);
 }
 
-/* vim:set foldmethod=marker expandtab: */
-
+/* vim:set foldmethod=marker: */
